#include "main.h"

extern u32 new_pad;

byte menu_bg_buf[480*272*3+0x36];
byte *menu_bg;

// by kwn
void save_config(void)
{
	char CfgPath[MAX_PATH];
	char *p;
	getDir(RinPath);
	strcpy(CfgPath, RinPath);
	p = strrchr(CfgPath, '/')+1;
	strcpy(p, "RIN.CFG");

	int fd;
	fd = sceIoOpen(CfgPath,O_CREAT|O_WRONLY|O_TRUNC, 0777);
	if(fd>=0){
		sceIoWrite(fd, &setting, sizeof(setting));
		sceIoClose(fd);
	}
}

// by kwn
void load_config(void)
{
	int i;
	char CfgPath[MAX_PATH];
	char *p;
	getDir(RinPath);
	strcpy(CfgPath, RinPath);
	p = strrchr(CfgPath, '/')+1;
	strcpy(p, "RIN.CFG");

	int fd;
	fd = sceIoOpen(CfgPath,O_RDONLY, 0777);
	if(fd>=0){
		sceIoRead(fd, &setting, sizeof(setting));
		sceIoClose(fd);
		if(setting.key_config[6]==0)
			setting.key_config[6] = CTRL_TRIANGLE;
		if(!_strcmp(setting.vercnf, "RINv1.1"))
			return;
	}

	strcpy(setting.vercnf, "RINv1.1");
	setting.screensize = 0;
	setting.vsync = 1;
	setting.sound = 1;
	setting.key_config[0] = CTRL_CIRCLE;
	setting.key_config[1] = CTRL_CROSS;
	setting.key_config[2] = CTRL_TRIANGLE;
	setting.key_config[3] = CTRL_SQUARE;
	setting.key_config[4] = CTRL_SELECT;
	setting.key_config[5] = CTRL_START;
	setting.key_config[6] = CTRL_LTRIGGER;
	for(i=7; i<16; i++)
		setting.key_config[i] = 0;
	setting.color[0] = 0x9063;
	setting.color[1] = RGB(85,85,95);
	setting.color[2] = RGB(105,105,115);
	setting.color[3] = 0xffff;
}

int save_sram(byte *buf,int size)
{
	int fd, tmp;
	int sram_tbl[]={1,1,1,4,16,8};
	
	fd = sceIoOpen(SavePath,O_CREAT|O_WRONLY|O_TRUNC, 0777);
	if(fd>=0){
		sceIoWrite(fd, buf, 0x2000*sram_tbl[size]);
		if ((rom_get_info()->cart_type>=0x0f)&&(rom_get_info()->cart_type<=0x13)){
			tmp=renderer_get_timer_state();
			sceIoWrite(fd, &tmp, 4);
		}
		sceIoClose(fd);
		return 1;
	}else
		return 0;
}

int load_sram(void)
{
	int fd;
	char *p;
	int ramsize;
	
	strcpy(SavePath, RomPath);
	p = strrchr(SavePath, '.');
	strcpy(p, ".sav");
	if((fd=sceIoOpen(SavePath, O_RDONLY, 0777))>=0){
		ramsize = sceIoRead(fd, sram_space, 16*0x2000+4);
		sceIoClose(fd);
		return ramsize;
	}
	
	strcpy(SavePath,RinPath);
	strcat(SavePath,"save/");
	strcat(SavePath,RomName);
	
	p = strrchr(SavePath, '.');
	strcpy(p, ".sav");
	if((fd=sceIoOpen(SavePath, O_RDONLY, 0777))>=0){
		ramsize = sceIoRead(fd, sram_space, 16*0x2000+4);
		sceIoClose(fd);
		return ramsize;
	}
	
	strcpy(SavePath,RinPath);
	strcat(SavePath,"SAVE/");
	strcat(SavePath,RomName);
	p = strrchr(SavePath,'.');
	strcpy(p, ".sav");
	return 0;
}

int save_state(void)
{
	int fd;
	char path[MAX_PATH], *p;
	
	strcpy(path, SavePath);
	p = strrchr(path, '.');
	strcpy(p, ".sv0");
	fd = sceIoOpen(path,O_CREAT|O_WRONLY|O_TRUNC, 0777);
	if(fd>=0){
		gb_save_state(fd);
		sceIoClose(fd);
		return 1;
	}else
		return 0;
}

int load_state(void){
	int fd;
	char path[MAX_PATH], *p;
	
	strcpy(path, SavePath);
	p = strrchr(path, '.');
	strcpy(p, ".sv0");
	fd = sceIoOpen(path,O_RDONLY, 0777);
	if(fd>=0){
		gb_restore_state(fd);
		sceIoClose(fd);
		return 1;
	}else
		return 0;
}

// by kwn
void load_menu_bg()
{
	char BgPath[256];
	char *p;
	getDir(RinPath);
	strcpy(BgPath, RinPath);
	p = strrchr(BgPath, '/')+1;
	strcpy(p, "RINMENU.BMP");

	int fd;
	fd = sceIoOpen(BgPath,O_RDONLY,0777);
	if(fd>=0){
		sceIoRead(fd, menu_bg_buf, 480*272*3+0x36);
		sceIoClose(fd);
		menu_bg = menu_bg_buf + 0x36;
	}else{
		menu_bg = 0;
	}
}

// by kwn
void BitBlt_menu_bg()
{
	unsigned char *vptr0;	//pointer to vram
	unsigned char *vptr;	//pointer to vram
	unsigned short x,y,yy,r,g,b,data;
	
	vptr0=pgGetVramAddr(0,0);
	for(y=0; y<272; y++){
		vptr=vptr0;
		for(x=0; x<480; x++){
			yy = 271 - y;
			r = *(menu_bg + (yy*480 + x)*3 + 2);
			g = *(menu_bg + (yy*480 + x)*3 + 1);
			b = *(menu_bg + (yy*480 + x)*3);
			data = (((b & 0xf8) << 7) | ((g & 0xf8) << 2) | (r >> 3));
			*(unsigned short *)vptr=data;
			vptr+=2;
		}
		vptr0+=1024;
	}
}

void rin_frame(const char *msg0, const char *msg1)
{
	if(menu_bg)
		BitBlt_menu_bg();
	else
		pgFillvram(setting.color[0]);
	mh_print(370, 0, " qhm Ver1.1 ", setting.color[1]);
	if(msg0!=0) mh_print(17, 14, msg0, setting.color[2]);
	pgDrawFrame(17,25,463,248,setting.color[1]);
	pgDrawFrame(18,26,462,247,setting.color[1]);
	if(msg1!=0) mh_print(17, 252, msg1, setting.color[2]);
}

void rin_colorconfig(void)
{
	enum
	{
		COLOR0_R=0,
		COLOR0_G,
		COLOR0_B,
		COLOR1_R,
		COLOR1_G,
		COLOR1_B,
		COLOR2_R,
		COLOR2_G,
		COLOR2_B,
		COLOR3_R,
		COLOR3_G,
		COLOR3_B,
		INIT,
		EXIT,
	};
	char tmp[256];
	int color[4][3];
	int sel=0, x, y, i;
	
	_memset(color, 0, sizeof(int)*4*3);
	for(i=0; i<4; i++){
		color[i][0] = setting.color[i]>>10 & 0x1F;
		color[i][1] = setting.color[i]>>5 & 0x1F;
		color[i][2] = setting.color[i] & 0x1F;
	}
	
	for(;;){
		readpad();
		if(new_pad & CTRL_CIRCLE){
			if(sel==EXIT){
				break;
			}else if(sel==INIT){
			}else{
				if(color[sel/3][sel%3]<31)
					color[sel/3][sel%3]++;
			}
		}else if(new_pad & CTRL_CROSS){
			if(color[sel/3][sel%3]>0)
				color[sel/3][sel%3]--;
		}else if(new_pad & CTRL_UP){
			if(sel!=0)	sel--;
			else		sel=EXIT;
		}else if(new_pad & CTRL_DOWN){
			if(sel!=EXIT)	sel++;
			else			sel=0;
		}
		
		for(i=0; i<4; i++)
			setting.color[i]=color[i][0]<<10|color[i][1]<<5|color[i][2]|0x8000;
		
		x = 2;
		y = 5;
		
		rin_frame(0, 0);
		
		pgPrint(x,y++,setting.color[3],"  COLOR0 R:");
		pgPrint(x,y++,setting.color[3],"  COLOR0 G:");
		pgPrint(x,y++,setting.color[3],"  COLOR0 B:");
		y++;
		pgPrint(x,y++,setting.color[3],"  COLOR1 R:");
		pgPrint(x,y++,setting.color[3],"  COLOR1 G:");
		pgPrint(x,y++,setting.color[3],"  COLOR1 B:");
		y++;
		pgPrint(x,y++,setting.color[3],"  COLOR2 R:");
		pgPrint(x,y++,setting.color[3],"  COLOR2 G:");
		pgPrint(x,y++,setting.color[3],"  COLOR2 B:");
		y++;
		pgPrint(x,y++,setting.color[3],"  COLOR3 R:");
		pgPrint(x,y++,setting.color[3],"  COLOR3 G:");
		pgPrint(x,y++,setting.color[3],"  COLOR3 B:");
		y++;
		pgPrint(x,y++,setting.color[3],"  INIT");
		pgPrint(x,y++,setting.color[3],"  Return to Main Menu");
		
		x=14; y=5;
		for(i=0; i<12; i++){
			if(i!=0 && i%3==0) y++;
			itoa(color[i/3][i%3], tmp);
			pgPrint(x,y++,setting.color[3],tmp);
		}
		
		x = 2;
		y = sel + 5 + sel/3;
		pgPutChar((x+1)*8,y*8,setting.color[3],0,127,1,0,1);
		
		pgScreenFlipV();
	}
}

// by kwn
void rin_keyconfig(void)
{
	enum
	{
		CONFIG_A = 0,
		CONFIG_B,
		CONFIG_RAPIDA,
		CONFIG_RAPIDB,
		CONFIG_SELECT,
		CONFIG_START,
		CONFIG_MENU,
		CONFIG_EXIT,
	};
	char msg[256];
	int sel=0, x, y, i;
	
	msg[0]=0;
	
	pgWaitVn(15);
	
	for(;;){
		readpad();
		if(new_pad & CTRL_CIRCLE){
			if(sel == CONFIG_EXIT){
				break;
			} else {
				setting.key_config[sel] = CTRL_CIRCLE;
			}
		}else if(new_pad & CTRL_SQUARE){
			if(sel != CONFIG_EXIT)
				setting.key_config[sel] = CTRL_SQUARE;
		}else if(new_pad & CTRL_TRIANGLE){
			if(sel != CONFIG_EXIT)
				setting.key_config[sel] = CTRL_TRIANGLE;
		}else if(new_pad & CTRL_CROSS){
			if(sel != CONFIG_EXIT)
				setting.key_config[sel] = CTRL_CROSS;
		}else if(new_pad & CTRL_START){
			if(sel != CONFIG_EXIT)
				setting.key_config[sel] = CTRL_START;
		}else if(new_pad & CTRL_SELECT){
			if(sel != CONFIG_EXIT)
				setting.key_config[sel] = CTRL_SELECT;
		}else if(new_pad & CTRL_LTRIGGER){
			if(sel != CONFIG_EXIT)
				setting.key_config[sel] = CTRL_LTRIGGER;
		}else if(new_pad & CTRL_RTRIGGER){
			if(sel != CONFIG_EXIT)
				setting.key_config[sel] = CTRL_RTRIGGER;
		}else if(new_pad & CTRL_LEFT){
			if(sel != CONFIG_EXIT && sel!=CONFIG_MENU)
				setting.key_config[sel] = 0;
		}else if(new_pad & CTRL_RIGHT){
			if(sel != CONFIG_EXIT && sel!=CONFIG_MENU)
				setting.key_config[sel] = 0;
		}else if(new_pad & CTRL_UP){
			if(sel!=0)	sel--;
			else		sel=CONFIG_EXIT;
		}else if(new_pad & CTRL_DOWN){
			if(sel!=CONFIG_EXIT)	sel++;
			else				sel=0;
		}
		
		rin_frame(msg, 0);
		
		x = 2;
		y = 5;
		
		pgPrint(x,y++,setting.color[3],"  A BUTTON       :");
		pgPrint(x,y++,setting.color[3],"  B BUTTON       :");
		pgPrint(x,y++,setting.color[3],"  A BUTTON(RAPID):");
		pgPrint(x,y++,setting.color[3],"  B BUTTON(RAPID):");
		pgPrint(x,y++,setting.color[3],"  SELECT BUTTON  :");
		pgPrint(x,y++,setting.color[3],"  START BUTTON   :");
		pgPrint(x,y++,setting.color[3],"  MENU BUTTON    :");
		y++;
		pgPrint(x,y++,setting.color[3],"  Return to Main Menu");
		
		for (i=0; i<7; i++){
			y = i + 5;
			if (setting.key_config[i] == CTRL_SQUARE)
				pgPrint(21,y,setting.color[3],"SQUARE");
			else if (setting.key_config[i] == CTRL_TRIANGLE)
				pgPrint(21,y,setting.color[3],"TRIANGLE");
			else if (setting.key_config[i] == CTRL_CIRCLE)
				pgPrint(21,y,setting.color[3],"CIRCLE");
			else if (setting.key_config[i] == CTRL_CROSS)
				pgPrint(21,y,setting.color[3],"CROSS");
			else if (setting.key_config[i] == CTRL_START)
				pgPrint(21,y,setting.color[3],"START");
			else if (setting.key_config[i] == CTRL_SELECT)
				pgPrint(21,y,setting.color[3],"SELECT");
			else if (setting.key_config[i] == CTRL_LTRIGGER)
				pgPrint(21,y,setting.color[3],"L");
			else if (setting.key_config[i] == CTRL_RTRIGGER)
				pgPrint(21,y,setting.color[3],"R");
			else 
				pgPrint(21,y,setting.color[3],"UNDEFINED");
		}
		
		y = sel + 5;
		if(sel >= CONFIG_EXIT)		y++;
		pgPutChar((x+1)*8,y*8,setting.color[3],0,127,1,0,1);
		
		pgScreenFlipV();
	}
	
}

void rin_menu(void)
{
	enum
	{
		SRAM_SAVE=0,
		STATE_SAVE,
		STATE_LOAD,
		SCREEN_SIZE,
		VSYNC,
		SOUND,
		COLOR_CONFIG,
		KEY_CONFIG,
		LOAD_ROM,
		CONTINUE,
	};
	int bSave, bBattery;
	char msg[256], path[256];
	int sel=0, x, y;
	int fd, romsize, ramsize;
	char *p;
	
	msg[0]=0;
	
	bBattery=rom_has_battery();
	if(!bBattery)
		sel = STATE_SAVE;
	
	for(;;){
		readpad();
		if(new_pad & CTRL_CIRCLE){
			if(sel == SRAM_SAVE){
				if(!save_sram(get_sram(), rom_get_info()->ram_size))
					strcpy(msg, "SRAM Save Failed");
				else
					strcpy(msg, "SRAM Saved Successfully");
			}else if(sel == STATE_SAVE){
				if(save_state())
					strcpy(msg, "State Saved Successfully");
				else
					strcpy(msg, "State Save Failed");
			}else if(sel == STATE_LOAD){
				if(load_state())
					strcpy(msg, "State Loaded Successfully");
				else
					strcpy(msg, "State Load Failed");
			}else if(sel == SCREEN_SIZE){
				setting.screensize++;
				if(setting.screensize>=4) setting.screensize=0;
			}else if(sel == VSYNC){
				setting.vsync = !setting.vsync;
			}else if(sel == SOUND){
				setting.sound = !setting.sound;
			}else if(sel == COLOR_CONFIG){
				rin_colorconfig();
			}else if(sel == KEY_CONFIG){
				rin_keyconfig();
			}else if(sel == LOAD_ROM){
				if(getFilePath(RomPath)==true){
					// by Smiths
					//Save SRAM before going back to menu
					if(bBattery)
						save_sram(get_sram(), rom_get_info()->ram_size);
					for(;;){
						/*
						bSave=0;
						strcpy(SavePath, RomPath);
						p = strrchr(SavePath, '.');
						strcpy(p, ".SAV");
						if((fd=sceIoOpen(SavePath, O_RDONLY, 0777))>=0){
							bSave=1;
						}else{
							p = strrchr(SavePath, '.');
							strcpy(p, ".sav");
							if((fd=sceIoOpen(SavePath, O_RDONLY, 0777))>=0){
								bSave=1;
							}
						}
						if(bSave && (fd=sceIoOpen(SavePath, O_RDONLY, 0777))>=0){
							ramsize = sceIoRead(fd, sram_space, 16*0x2000+4);
							sceIoClose(fd);
						}
						*/
						
						p = strrchr(RomPath,'/');
						strcpy(RomName, p+1);
						ramsize = load_sram();
						if(ramsize)
							bSave = 1;
						
						fd=sceIoOpen(RomPath, O_RDONLY, 0777);
						romsize = sceIoRead(fd, rom_image, 8*1024*1024);
						sceIoClose(fd);
						
						gb_init();
						gb_load_rom(rom_image, romsize, bSave?sram_space:0, bSave?ramsize:0);
						break;
					}
					break;
				}
			}else if(sel == CONTINUE){
				break;
			}
		}else if(new_pad & CTRL_CROSS){
			break;
		}else if(new_pad & CTRL_UP){
			if(sel==STATE_SAVE && !bBattery)
				sel=CONTINUE;
			else if(sel!=0)
				sel--;
			else
				sel=CONTINUE;
		}else if(new_pad & CTRL_DOWN){
			if(sel!=CONTINUE)	sel++;
			else				sel=0;
		}else if(new_pad & CTRL_LEFT){
			if(sel>KEY_CONFIG) sel=SCREEN_SIZE;
			else if(sel>SRAM_SAVE) sel=SRAM_SAVE;
		}else if(new_pad & CTRL_RIGHT){
			if(sel<SCREEN_SIZE) sel=SCREEN_SIZE;
			else if(sel<LOAD_ROM) sel=LOAD_ROM;
		}
		
		if(!bBattery && sel==SRAM_SAVE)
			sel = STATE_SAVE;
		
		rin_frame(msg, 0);
		
		x = 2;
		y = 5;
		
		if(bBattery)
			pgPrint(x,y++,setting.color[3],"  SRAM SAVE");
		else
			pgPrint(x,y++,setting.color[0],"  SRAM SAVE");
		pgPrint(x,y++,setting.color[3],"  STATE SAVE");
		pgPrint(x,y++,setting.color[3],"  STATE LOAD");
		y++;
		
		if(setting.screensize==0)
			pgPrint(x,y++,0xffff,"  SCREEN SIZE: x1");
		else if(setting.screensize==1)
			pgPrint(x,y++,0xffff,"  SCREEN SIZE: x1.5(filtered)");
		else if(setting.screensize==2)
			pgPrint(x,y++,0xffff,"  SCREEN SIZE: x2(uncropped)");
		else if(setting.screensize==3)
			pgPrint(x,y++,0xffff,"  SCREEN SIZE: x2(double-size)");
		
		if(setting.vsync)
			pgPrint(x,y++,setting.color[3],"  VSYNC:ON");
		else
			pgPrint(x,y++,setting.color[3],"  VSYNC:OFF");
		
		if(setting.sound)
			pgPrint(x,y++,setting.color[3],"  SOUND:ON");
		else
			pgPrint(x,y++,setting.color[3],"  SOUND:OFF");
		
		pgPrint(x,y++,setting.color[3],"  COLOR CONFIG");
		pgPrint(x,y++,setting.color[3],"  KEY CONFIG");
		y++;
		pgPrint(x,y++,setting.color[3],"  Back to ROM list");
		pgPrint(x,y++,setting.color[3],"  Continue");
		
		y = sel + 5;
		if(sel >= SCREEN_SIZE)	y++;
		if(sel >= LOAD_ROM)		y++;

		pgPutChar((x+1)*8,y*8,setting.color[3],0,127,1,0,1);
		
		pgScreenFlipV();
	}
	
	save_config();
	
	pgFillvram(0);
	pgScreenFlipV();
	pgFillvram(0);
	pgScreenFlipV();
	pgWaitVn(10);
	_memset(&paddata, 0x00, sizeof(paddata));
}
